
;--------------------------------------------------------------------------------------------------------------
; str_eq0
; -------------------------------------------------------------------------------------------------------------
; Comparing 2 strings - 5000 bytes long, using a parallel approach.
; One scan goes from the beginning of the string toward the end.
; A second scan goes from the end of the string backward to the start.  
;--------------------------------------------------------------------------------------------------------------
; Assumptions:
;      1. Strings are of the same length and NULL terminated
;      2. We don't know the lenght of the strings
;      3. We need to know if the strings are equal or different
;--------------------------------------------------------------------------------------------------------------
; Method proposed by frktons
;--------------------------------------------------------------------------------------------------------------
; 22 june 2010 - masmforum
;--------------------------------------------------------------------------------------------------------------


include \masm32\include\masm32rt.inc

.686

include \masm32\macros\timers.asm

	
		
LOOP_COUNT	= 10000

.data

ALIGN 16


Src1	db "This is a string, 100 characters long, that serves for a variety of purposes, such as testing algos?"
   REPEAT 48
	db "This is a string, 100 characters long, that serves for a variety of purposes, such as testing algos?"
   ENDM
	db "This is a string, 100 characters long, that serves for a variety of purposes, such as testing algos!", 0	


ALIGN 16

Src2	db "This is a string, 100 characters long, that serves for a variety of purposes, such as testing algos?"
   REPEAT 48
	db "This is a string, 100 characters long, that serves for a variety of purposes, such as testing algos?"
   ENDM
	db "This is a string, 100 characters long, that serves for a variety of purposes, such as testing algos!", 0	

LenSrc      dd 0 
Spare_bytes dd 0

.code

start:	push 1
	call ShowCpu	                                     ; prints the brand string and returns SSE level
	counter_begin LOOP_COUNT, HIGH_PRIORITY_CLASS       ; start counting the cycles 


        lea ebx, Src1           ; We find out the length of the strings
        mov eax, len(ebx)       ; and we get the number of cycles the loop
        mov LenSrc, eax         ;
        shr eax, 3              ; has to do to scan the whole strings
        mov ebx, eax            ; length of the string / 8
        shl ebx, 3              ; and if the division has got a remainder
        mov ecx, LenSrc         ; we do an extra cycle to cover the
        sub ecx, ebx            ; spare bytes. 
        jz  loop_cmp            ; If no remainder we go straigth to the scan loop.
        add eax, 1              ; An extra cycle to cover the spare bytes.
        mov Spare_bytes, ecx    ; These bytes will be checked in another routine

loop_cmp:

        lea ebx, Src1
        mov ecx, LenSrc
        add ecx, ebx
        sub ecx, 4
        lea edx, Src2
        mov edi, LenSrc
        add edi, edx
        sub edi, 4

compare:

        mov esi, [ebx]
        cmp esi, [edx]
        jnz str_ne

        mov esi, [ecx]
        cmp esi, [edi]
        jnz str_ne

        add ebx, 4
        add edx, 4

        sub ecx, 4
        sub edi, 4

        dec eax
        jnz compare
        

str_ne:

       	counter_end

	print str$(eax), 9, "cycles for Parallel scan, 5000 bytes", 13, 10

	
       inkey chr$(13, 10, "--- ok ---", 13)
        
	exit

push 1

ShowCpu proc		; mode:DWORD
COMMENT @ Usage: 
  push 0, call ShowCpu	; simple, no printing, just returns SSE level
  push 1, call ShowCpu	; prints the brand string and returns SSE level@
  pushad
  sub esp, 80	; create a buffer for the brand string
  mov edi, esp		; point edi to it
  xor ebp, ebp
  .Repeat
  	lea eax, [ebp+80000002h]
	db 0Fh, 0A2h		; cpuid 80000002h-80000004h
	stosd
	mov eax, ebx
	stosd
	mov eax, ecx
	stosd
	mov eax, edx
	stosd
	inc ebp
  .Until ebp>=3
  push 1
  pop eax
  db 0Fh, 0A2h			; cpuid 1
  xor ebx, ebx					; CpuSSE
  xor esi, esi					; add zero plus the carry flag
  bt edx, 25					; edx bit 25, SSE1
  adc ebx, esi
  bt edx, 26					; edx bit 26, SSE2
  adc ebx, esi
  bt ecx, esi					; ecx bit 0, SSE3
  adc ebx, esi
  bt ecx, 9					; ecx bit 9, SSE4
  adc ebx, esi
  dec dword ptr [esp+4+32+80]			; dec mode in stack
  .if Zero?
	mov edi, esp				; restore pointer to brand string
  	.Repeat				
		.Break .if byte ptr [edi]!=32	; mode was 1, so show a string but skip leading blanks
		inc edi
	.Until 0
	print edi				; CpuBrand
	.if ebx
		print chr$(32, 40, "SSE")		; info on SSE level, 40=(
		print str$(ebx), 41, 13, 10		; 41=)
	.endif
  .endif
  add esp, 80			; discard brand buffer (after printing!)
  mov [esp+32-4], ebx		; move ebx into eax stack position - returns eax to main for further use
  popad
  ret 4
ShowCpu endp

end start

